VG_HECM <- function(y, mu, sigma2, gamma, nu, Delta, eps, maxiter)
{

  # This function estimates the parameters of the Variance-gamma distirbution using the HECM algorithm; see
  # Bee, M., Dickson, M.M. and Santi, F. (2017), "Likelihood-based Risk Estimation for Variance-Gamma Models",
  # DEM working paper 2017/03, http://www.economia.unitn.it/71/dem-working-papers. 
  
  # Inputs:  
  # y: numeric vector of data;
  # mu: starting value of mu
  # sigma2: starting value of sigma^2;
  # gamma: starting value of gamma;
  # nu: starting value of nu;
  # Delta: value of Delta to be used n the E step;
  # eps: tolerance
  # maxiter: maximum number of iterations.
  
  # Output:
  
  # parsHECM: (6x1) numeric vector; the forst 4 vaues are  the estimates of mu, sigma^2, gamma and nu,
  # the fifth value is the optimized log-likelihood and the last value is the number of iterations.
  
  # Usage: 
  
  # library(tseries)
  # library(zoo)
  # library(lsr)
  # source('VG_HECM.R')
  # price<- get.hist.quote(instrument = "^GSPC",start = "2008-01-02",end = "2008-12-31", quote="AdjClose")
  # r <- diff(log(price))
  # dem_r <- r - mean(r)
  # y <- coredata(dem_r)
  # y <- as.vector(y)
  # nus = 2
  # gammas = 0
  # Delta = 1.e-2
  # sigmas = aad(y) # mean absolute deviation
  # mus = mean(y)
  # eps <- 10^(-8)                                   # convergence criterion
  # maxiter = 2000
  # parsHECM = VG_HECM(y, mus, sigmas^2, gammas, nus, Delta, eps, maxiter)
  
source('VG_MCECM.r')
source('VG_ECME.r')
change <- 1                           # initial test value for convergence
parsMCECM = VG_MCECM(y, mu, sigma2, gammap, nu, Delta, eps, maxiter)
mu <- parsMCECM[1]
sigma2 <- parsMCECM[2]
gammap <- parsMCECM[3]
nu <- parsMCECM[4]
res = VG_ECME(y, mu, sigma2, gammap, nu, Delta, eps, maxiter)
res[6] <- res[6] + parsMCECM[6] - 1
res
}